function res=randclade

%====================================================================================================================
%Part of this code is based on code of McShea, D. 1994. Mechanisms of large-scale evolutionary trends. 
%Evolution 48:1747-1763. I am grateful to him for providing me with the initial code.
%
%This code simulated evolution of monophyletic clade in morphospace. 
%Dimensions are a mean of zero, a variance equal to the unity and no covariance. 
%Probabilities and magnitude of change are equal among dimensions.
%Probability of change by anagenis is equal to the probabilities of stasis.
%Probabilities of increases and decreases when anagenis or branching occurs, are equal => passive system
%
%Two Monophyletic subclades are extracted and two are randomly chosen.
%
%Steps (0.2 myr) are grouped in 14 bins representing 5 myr. => 70 myr (pres_clade.m)
%Data set is altered to simulated incomplete, time-averaged fossil record with constant probability of preservation.
%Preserved occurences of species are regrouped within bins. The morphological mean of a species in preserved samples
%is considered as the species' morphology. At the end, a correction for Lazarus taxa is incorporated in occurrence file.
%All subroutines are integrated in this file.
%
%Input for origination and extinction rates are based on results of Alroy (2000 in Paleobiology)
%Input for preservation rates are based on Raup and Foote (1996, Paleobiology) results.
%
%Results are contain and save in the res MAT files with fields:
%res = 
%
%             morpho: [species_pres x dims  double]
%         occurrence: [species_pres x nbins double]
%    occur_LAZARcorr: [species_pres x nbins double]   : occurrence field corrected for Lazarus taxa
%             LazarT: [0 0 0 0 0 0 1 0 0 0 0 0 0 0] 1 x nbins: 0 no Lazarus taxa; N Lazarus taxa
%          group_all: [species_pres x nsubclade    double]
%       group_select: [species_pres x 1+nsclselect  double]  => the whole + clrand
%         clade: [species x dims x time double]
%       SSclade: [  1 x nsubclade    struct]
%        clrand: [  nsclselect x 1     double]
%
%Subroutines pres_clade.m is provided also separatly for test various alteration (bin, probability of preservation...)
%on the same simulation (res.clade res.SSclade and res.clrand)
%
%Program used in N. Navarro. MDA: a Matlab-based program for Morphospace-Disparity Analysis. Computers & Geosciences.
%for construct data for examples in this introductory paper of the MDA program.
%For this:
% data_matrix=res.morpho;
% save data_matrix.txt data_matrix -ascii
%and similarly for group and occurrence files.
%
%
%(C) N. Navarro 2002.
%====================================================================================================================


diversity=[];
nber_dims=2;
sa=ones(1,nber_dims);
pcha=0.5; %probability of stasis = 1-pcha
%probilities of increase or decrease at the time of anagenesis
%if pi>>pd => driven system otherwise passive
pia=0.5;   
pda=0.5; 
sb=ones(1,nber_dims);

%probability of change when speciation occurs
pchb=1;
%probabilities of increase or decrease when change
%if pi>>pd => driven system otherwise passive
pib=0.5;
pdb=0.5;
%no boundary

run=1;
k=1;
nrun_max=25;
nrun=0;
species=1;
diversity(1)=1;
while nrun<nrun_max
      clade=[];
      [species nber_morphodim time]=size(clade);
      nber_morphodim=nber_dims;
      branch=[];
      %pb and pe according to data of Alroy 2000 (mean on 70 Ma)
      pb=0.059; 
      pe=0.055;
      %70 Ma with 1 step = 200.000yrs: end Late Cretaceous-Present day =>350 levels
      time=1+(70/0.2);
      k=1;
      species=1;
      origin=zeros(1,nber_morphodim);
      clade(1,:,1)=origin(1,:);
      branch(1,1)=0;
for t=2:time
   extinction=0;
   origination=0;
   for i=1:species
      if isnan(clade(i,:,t-1));
         clade(i,:,t)=nan;
      else
         %anagenesis change
         ch_anag=change(sa,pcha,pia,pda,nber_morphodim);
         clade(i,:,t)=origin(i,:)+ ch_anag;
         origin(i,:)=clade(i,:,t);
         %branching event
         if rand<pb
            species=species+1;
            origination=origination+1;
             ch_branch=change(sb,pchb,pib,pdb,nber_morphodim);
             clade(species,:,t)=origin(i,:)+ ch_branch;
             clade(species,:,1:t-1)=nan;
             branch(species,1)=i;
             origin(species,:)=clade(species,:,t);
         end;
         %extinction event
         if rand<pe
             clade(i,:,t)=nan;
             extinction=extinction+1;
         end;
       end;
    end;
    diversity(t)=diversity(t-1)-extinction+origination;
 end;
 if diversity(time)>=nrun_max
    % Subdividing clade into subclade
    SSclade=subclade(branch,species);
    hh=0;
 for ii=1:SSclade(1).ncld
    hh=hh+1;
    SSclade(hh).species=SSclade(ii).species;
    SSclade(hh).morpho=clade(SSclade(hh).species,:,:);
    indexpday=1-isnan(SSclade(hh).morpho(:,1,time));
    if sum(indexpday)>1
       SSclade(hh).pday=SSclade(hh).morpho(find(indexpday==1),:,time);
       SSclade(hh).mean=mean(SSclade(hh).pday);
           SSclade(hh).morpho=[];
           SSclade(hh).pday=[];
    if ii>1
       %selection of just subclade contain in upper tail 
       %(considering that upper tail is defined by a subclade mean > to clade mean)
       aa=0;
       for nd=1:2
          if SSclade(hh).mean(1,nd)>SSclade(1).mean(1,nd)
             aa=1;
          end;
       end;
       SSclade(hh).mean=[];
       if aa==1         
          hh=hh;
       else
          hh=hh-1;
       end;
    end;
    else
       hh=hh-1;
    end;
    indexpday=[];
    indexsp=[];
 end;
 if hh>1
    nrun=nrun_max;
       fprintf('\n Number of run: %i\t',run);
       fprintf('\n End-point diversity: %i\t',diversity(time));
       fprintf('\n Number of subclade: %i\t',hh);
    end;
 else 
    run=run+1;  
 end;
end;
clear extinction origination branch origin;

%Random choice of N monophyletic subclade
nc=2;
clrand=randomCLADE(SSclade,nc,hh);
fprintf('\n subclade chosen: %i',clrand);
fprintf('\n');

if length(clrand)<nc
   nc=length(clrand); %modify if paraphyletic 
end;

figure(1);
T=1:time;
subplot(411); 
plot(diversity,'k:');
hold on;
subplot(312);
   axis1=clade(:,1,:);
   axis1=permute(axis1,[1,3,2]);
   plot(T,axis1,'k-');
   hold on;
   coul=['b','r','y'];
   subplot(313);
   axis2=clade(:,2,:);
   axis2=permute(axis2,[1,3,2]);
   plot(T,axis2,'k-');
   hold on;

for ii=1:nc
   subplot(312);
   clad=clade((SSclade(clrand(ii,1)).species),:,:);
   axis1b=clad(:,1,:);
   axis1b=permute(axis1b,[1,3,2]);
   plot(T,axis1b,coul(ii));
   hold on;
   subplot(313);
   axis2b=clad(:,2,:);
   axis2b=permute(axis2b,[1,3,2]);
   plot(T,axis2b,coul(ii));
   hold on;
end;

%----------------------------
%End of Simulation
%----------------------------
%Suppress preservation subroutine if you want save then stop program 
%at the end of the simulation
%------------------------------------------------------------
%Subroutine for preservation, time-averaging and Lazarus taxa
%------------------------------------------------------------
res=pres_clade(clade,SSclade,clrand);
%------------------------------------------------------------

%Results of simulation
res.clade=clade;
res.SSclade=SSclade;
res.clrand=clrand;

%save results of simulation (clade and subclades and clrand) 
%and preservation results in structure MAT file res
save res res 

%----------------------------------------------
function ch=change(ss, pchb, pinc, pdec,ndims)

chang=randn(1,ndims).*ss;
for n=1:ndims
pch=rand;
   if pch<pchb  
      prob=rand;
      if prob<pinc
         ch(1,n)=abs(chang(1,n));
      elseif prob>(pinc+pdec)
         ch(1,n)=0;
      else  
         ch(1,n)=-1.*abs(chang(1,n));
      end;
   else
      ch(1,n)=0;
   end;
end;
%--------------------------------------------
function subclade=subclade(data,species)
ncld=0;
for jj=1:species
      spdaughter=cat(1,jj,find(data==jj));
      spdaugh=length(spdaughter);
      ncld=ncld+1;
      ii=1;
      kk=1;
      if spdaugh>1
      while kk==1
          ii=ii+1;
          spdaughter=cat(1,spdaughter,find(data==spdaughter(ii,1)));
          spdaugh=length(spdaughter);
          if ii==spdaugh
             kk==0;
             break
          end;
      end;
      end;
      if spdaugh>=species/10
         subclade(ncld).species=spdaughter;
      else
         ncld=ncld-1;
      end;
      spdaugh=0;
      spdaughter=[];
      subclade(1).ncld=ncld;
end;
%---------------------------------------------
function clrand=randomCLADE(SSclade,nc,hh)
%if a subclade choose appartain to another choose, this latter
%is suppress and just the most derived clade is conserved.
%Permit to conserved only monophyletic clade (suppress paraphyletic)
%avoid to choose the main clade

kk=100000;
nsc=nc;
t=0;
max=nchoosek(SSclade(1).ncld,nc);
clrand=[];
while kk~=hh
   t=t+1;
   clrand=sort(cat(1,clrand,ceil(rand(nsc,1)*hh)));
   commun=zeros(3,2);
 for ii=1:nc-1
    for jj=ii+1:nc 
        com=find(SSclade(clrand(jj,1)).species(1,1)==SSclade(clrand(ii,1)).species);
        commun(jj,ii)=1-isempty(com);
    end;
 end;
 if sum(sum(commun))~=0
    [l p]=find(commun==1);
    replac=clrand(p,1);
    nsc=length(replac);
    conserv=ones(nc,1);
    for ii=1:nc
       for jj=1:nsc
          if p(nsc)==ii
             conserv(ii,1)=0;
          end;
       end;
    end;
    clrand=clrand(find(conserv==1),1);
 else
    kk=hh;
 end;
 if t==max
    warning('choice of paraphyletic subclade can be probable!!!'); 
    break
 end;
end;

%==============================================================
%Subroutines for preservation, time averaging and Lazarus taxa.
%==============================================================
function res=pres_clade(clade,SSclade,clrand)

[species dim time] = size(clade);
[l hh]=size(SSclade);
%rate of preservation per interval (200.000 yrs) 
%=> 0.25 per 0.7 myrs (=0.35 Lma)[Foote and Raup 1996 for Cenozoic mammals]
p_pres=0.07;
%Bin of 5 myrs (lower limit) convert into steps.
Lbin=[1	26	51	76	101	126	151	176	201	226	251	276	301	326];  
range=Lbin(2)-Lbin(1);
nbins=length(Lbin);

occurence=1-isnan(clade(:,1,:));
occurence=reshape(occurence,species,time);

for jj=1:time
  for ii=1:species
     if occurence(ii,jj)==1
        preservation=rand;
        if preservation<p_pres
           occurence(ii,jj)=1;
        else
           occurence(ii,jj)=0;
           clade(ii,:,jj)=ones(1,2)*nan;
        end;
     end;
  end;
end;

for nb=1:1:nbins
   Llimit=Lbin(nb);
   if nb==nbins
      ULimit=Lbin(nb)+range;
   else
      ULimit=Lbin(nb+1)-1;
   end;
   indsteps=Llimit:ULimit;
   occbin=occurence(:,indsteps);
   for ii=1:species
      if sum(occbin(ii,:))>=1
         occur(ii,nb)=1;
      for j=1:dim
         mm=clade(ii,j,:);
         mmm=mm(find(1-isnan(mm)));
         if isempty(mmm)
            c=nan;
         else
            c=mean(mmm);
         end;
         MeanM(ii,j)=c;
      end;
      else
         occur(ii,nb)=0;
      end;
   end;      
end;
somm=sum(occur,2);
indexsauv=find(somm~=0);
yy=length(indexsauv);
res.morpho=MeanM(indexsauv,:);
clear MeanM;

res.occurrence=occur(indexsauv,:);
clear occur;
res.occur_LAZARcorr=lazarT(res.occurrence);
res.LazarT=sum(res.occur_LAZARcorr-res.occurrence);

group=zeros(species,hh);
for h=1:hh
   index=SSclade(h).species;
   kk=length(index);
   for sp=1:species
      for jj=1:kk
      if sp==index(jj)
         group(sp,h)=1;
      end;
      end;
   end;
end;
res.group_all=group(indexsauv,:);
clselect=cat(1,1,clrand);
res.group_select=res.group_all(:,clselect');

%-----------------------------------------
%subroutine for Lazarus taxa
%-----------------------------------------
function cc=lazarT(occurrence)
[sp time]=size(occurrence);
cc=occurrence;
for ii=1:sp
   kk=find(occurrence(ii,:));
   if length(kk)>2
      cc(ii,kk(1):kk(end))=ones(1,kk(end)-kk(1)+1);
   end;
end;
   



